@@ -8,7 +8,7 @@ use rtlola_hir::{CompleteMode, RtLolaHir};
 use rtlola_parser::ast::WindowOperation;
 
 use crate::mir;
-use crate::mir::{InstanceTemplate, Mir, PacingType, SpawnTemplate};
+use crate::mir::{CloseTemplate, FilterTemplate, InstanceTemplate, Mir, PacingType, SpawnTemplate};
 
 impl Mir {
     /// Generates an Mir from a complete Hir.
@@ -34,24 +34,21 @@ impl Mir {
             "SRefs need to enumerated from 0 to the number of streams"
         );
 
-        let outputs = hir
-            .outputs()
-            .map(|o| {
-                let sr = o.sr();
-                mir::OutputStream {
-                    name: o.name.clone(),
-                    ty: Self::lower_value_type(&hir.stream_type(sr).value_ty),
-                    expr: Self::lower_expr(&hir, hir.expr(sr)),
-                    instance_template: Self::lower_instance_template(&hir, sr),
-                    accesses: hir.direct_accesses(sr),
-                    accessed_by: hir.direct_accessed_by(sr),
-                    aggregated_by: hir.aggregated_by(sr),
-                    memory_bound: hir.memory_bound(sr),
-                    layer: hir.stream_layers(sr),
-                    reference: sr,
-                }
-            })
-            .collect::<Vec<mir::OutputStream>>();
+        let outputs = hir.outputs().map(|o| {
+            let sr = o.sr();
+            mir::OutputStream {
+                name: o.name.clone(),
+                ty: Self::lower_value_type(&hir.stream_type(sr).value_ty),
+                expr: Self::lower_expr(&hir, hir.expr(sr)),
+                instance_template: Self::lower_instance_template(&hir, sr),
+                accesses: hir.direct_accesses(sr),
+                accessed_by: hir.direct_accessed_by(sr),
+                aggregated_by: hir.aggregated_by(sr),
+                memory_bound: hir.memory_bound(sr),
+                layer: hir.stream_layers(sr),
+                reference: sr,
+            }
+        });
         let (trigger_streams, triggers): (Vec<mir::OutputStream>, Vec<mir::Trigger>) = hir
             .triggers()
             .sorted_by(|a, b| Ord::cmp(&a.sr(), &b.sr()))
@@ -80,7 +77,6 @@ impl Mir {
             })
             .unzip();
         let outputs = outputs
-            .into_iter()
             .chain(trigger_streams.into_iter())
             .sorted_by(|a, b| Ord::cmp(&a.reference, &b.reference))
             .collect::<Vec<_>>();
@@ -195,15 +191,57 @@ impl Mir {
         let spawn_cond = hir_spawn_condition.map(|_| Self::lower_expr(hir, &spawn.1));
         let spawn_target = hir_spawn_target.map(|target| Self::lower_expr(hir, target));
         let filter = hir.filter(sr).map(|_| Self::lower_expr(hir, &filter));
+        let filter_pacing = hir
+            .filter(sr)
+            .map(|expr| {
+                match hir.expr_type(expr.id()).pacing_ty {
+                    ConcretePacingType::Event(ac) => PacingType::Event(Self::lower_activation_condition(&ac)),
+                    ConcretePacingType::FixedPeriodic(freq) => PacingType::Periodic(freq),
+                    ConcretePacingType::Constant => PacingType::Constant,
+                    ConcretePacingType::Periodic => {
+                        unreachable!("Ensured by pacing type checker")
+                    },
+                }
+            })
+            .unwrap_or(PacingType::Constant);
         let close = hir.close(sr).map(|_| Self::lower_expr(hir, &close));
+        let close_pacing = hir
+            .close(sr)
+            .map(|expr| {
+                match hir.expr_type(expr.id()).pacing_ty {
+                    ConcretePacingType::Event(ac) => PacingType::Event(Self::lower_activation_condition(&ac)),
+                    ConcretePacingType::FixedPeriodic(freq) => PacingType::Periodic(freq),
+                    ConcretePacingType::Constant => PacingType::Constant,
+                    ConcretePacingType::Periodic => {
+                        unreachable!("Ensured by pacing type checker")
+                    },
+                }
+            })
+            .unwrap_or(PacingType::Constant);
+        let close_self_ref = hir
+            .close(sr)
+            .map(|expr| {
+                matches!(
+                    hir.expr_type(expr.id()).spawn.0,
+                    ConcretePacingType::Event(_) | ConcretePacingType::FixedPeriodic(_)
+                )
+            })
+            .unwrap_or(false);
         InstanceTemplate {
             spawn: SpawnTemplate {
                 target: spawn_target,
                 pacing: spawn_pacing,
                 condition: spawn_cond,
             },
-            filter,
-            close,
+            filter: FilterTemplate {
+                target: filter,
+                pacing: filter_pacing,
+            },
+            close: CloseTemplate {
+                target: close,
+                pacing: close_pacing,
+                has_self_reference: close_self_ref,
+            },
         }
     }
 
@@ -391,7 +429,7 @@ impl Mir {
             ArithLogOp::BitAnd => mir::ArithLogOp::BitAnd,
             ArithLogOp::BitOr => mir::ArithLogOp::BitOr,
             ArithLogOp::BitNot => mir::ArithLogOp::BitNot,
-            ArithLogOp::Shl => mir::ArithLogOp::Shr,
+            ArithLogOp::Shl => mir::ArithLogOp::Shl,
             ArithLogOp::Shr => mir::ArithLogOp::Shr,
             ArithLogOp::Eq => mir::ArithLogOp::Eq,
             ArithLogOp::Lt => mir::ArithLogOp::Lt,
@@ -444,12 +482,9 @@ impl Mir {
 
 #[cfg(test)]
 mod tests {
-    use std::path::PathBuf;
-
     use num::rational::Rational64 as Rational;
     use num::FromPrimitive;
     use rtlola_parser::ParserConfig;
-    use rtlola_reporting::Handler;
     use uom::si::frequency::hertz;
     use uom::si::rational64::Frequency as UOM_Frequency;
 
@@ -457,11 +492,10 @@ mod tests {
     use crate::mir::IntTy::Int8;
 
     fn lower_spec(spec: &str) -> (RtLolaHir<CompleteMode>, mir::RtLolaMir) {
-        let handler = Handler::new(PathBuf::new(), spec.into());
         let ast = ParserConfig::for_string(spec.into())
             .parse()
-            .unwrap_or_else(|e| panic!("{}", e));
-        let hir = rtlola_hir::fully_analyzed(ast, &handler).expect("Invalid AST:");
+            .unwrap_or_else(|e| panic!("{:?}", e));
+        let hir = rtlola_hir::fully_analyzed(ast).expect("Invalid AST:");
         (hir.clone(), Mir::from_hir(hir))
     }
 
@@ -584,14 +618,14 @@ mod tests {
             PacingType::Periodic(UOM_Frequency::new::<hertz>(Rational::from_u8(1).unwrap()))
         );
         assert!(matches!(
-            mir_d.instance_template.filter,
+            mir_d.instance_template.filter.target,
             Some(mir::Expression {
                 kind: mir::ExpressionKind::ArithLog(mir::ArithLogOp::Eq, _),
                 ty: _,
             })
         ));
         assert!(matches!(
-            mir_d.instance_template.close,
+            mir_d.instance_template.close.target,
             Some(mir::Expression {
                 kind: mir::ExpressionKind::ArithLog(mir::ArithLogOp::Eq, _),
                 ty: _,
@@ -633,6 +667,19 @@ mod tests {
         assert_eq!(mir.triggers.len(), 1);
     }
 
+    #[test]
+    fn test_instance_window() {
+        let spec = "input a: Int32\n\
+        output b(p: Bool) spawn with a == 42 := a\n\
+        output c @1Hz := b(false).aggregate(over: 1s, using: sum)";
+        let (_, mir) = lower_spec(spec);
+
+        let expr = mir.outputs[1].expr.kind.clone();
+        assert!(
+            matches!(expr, mir::ExpressionKind::StreamAccess {target: _, parameters: paras, access_kind: mir::StreamAccessKind::SlidingWindow(_)} if paras.len() == 1)
+        );
+    }
+
     #[test]
     fn test_cast_lowering() {
         let spec = "input a: Int64\n\
